/***********************************************************************/
/*                                                                     */
/* RIGI Utilities --- Copyright (C) 1996-1999                          */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Author: Johannes Martin (jmartin@csr.csc.uvic.ca)                   */
/*                                                                     */
/***********************************************************************/
class CSImplementationParser: public CS_DepthFirstVisitor {
  CodeStoreParser& parser;
  CS_Implementation& impl;

  virtual void visitCastExpression(CS_CastExpression &ce);
  virtual void visitFunctionCallExpression(CS_FunctionCallExpression &fce);
  virtual void visitMemberExpression(CS_MemberExpression& me);
  virtual void visitNameExpression(CS_NameExpression &ne);
  virtual void visitLexicalBlockStatement(CS_LexicalBlockStatement &lbs);
public:
  CSImplementationParser(CodeStoreParser& _parser, CS_Implementation& _impl) : parser(_parser), impl(_impl) { }
};

void CSImplementationParser::visitCastExpression(CS_CastExpression &ce) {
  CS_TypeDescriptor& sourceType = ce.expression().typeDescriptor();
  CS_TypeDescriptor& destType   = ce.typeDescriptor();
  if (!parser.mapsToPredefinedType(sourceType) && !parser.mapsToPredefinedType(destType))
    parser.emitIsCastTo(parser.toString(sourceType),
                        parser.toString(destType),
                        &ce.sourceLocation());
  CS_DepthFirstVisitor::visitCastExpression(ce);
}

void CSImplementationParser::visitFunctionCallExpression(CS_FunctionCallExpression &call) {
  CS_Expression &funcExpr = call.functionExpression();
  CS_String dest;
  CS_Expression::Kind k;
  switch ((k = funcExpr.expressionKind())) {
    case CS_Expression::IsName:
      if (!funcExpr.asNameExpression()->declaration())
        return;
      dest = UniqueIdentifier::idForDeclaration(*funcExpr.asNameExpression()->declaration());
      break;
    case CS_Expression::IsFunctionCast:
      dest = UniqueIdentifier::idForDeclaration(funcExpr.asFunctionCastExpression()->functionCastTypeDescriptor().declaration());
      return;
    case CS_Expression::IsMember:
      if (!funcExpr.asMemberExpression()->declaration())
        return;
      dest = UniqueIdentifier::idForDeclaration(*funcExpr.asMemberExpression()->declaration());
      break;
    case CS_Expression::IsParentheses:
    case CS_Expression::IsUnary:
      return;
    default:
      cerr << "Unexpected function expression kind:" << k << endl;
      cerr << "call to " << funcExpr.signature() << endl;
#ifdef DEBUG
      dest = funcExpr.signature();
#else
      return;
#endif
  }
  parser.emitCalls(UniqueIdentifier::idForDeclaration(*impl.declaration()), dest, &call.sourceLocation());
  CS_DepthFirstVisitor::visitFunctionCallExpression(call);
}

void CSImplementationParser::visitMemberExpression(CS_MemberExpression& me) {
  CS_Expression& e = me.expression();
  if (e.state() >= CS_Implementation::analyzed)
    parser.emitAccesses(UniqueIdentifier::idForDeclaration(*impl.declaration()),
                        parser.toString(e.typeDescriptor()), &me.sourceLocation());
  CS_DepthFirstVisitor::visitMemberExpression(me);
}

void CSImplementationParser::visitNameExpression(CS_NameExpression &ne) {
  CS_Declaration *d = ne.declaration();
  if (d)
    parser.emitReferences(UniqueIdentifier::idForDeclaration(*impl.declaration()),
                          UniqueIdentifier::idForDeclaration(*d), &ne.sourceLocation());
  CS_DepthFirstVisitor::visitNameExpression(ne);
}

void CSImplementationParser::visitLexicalBlockStatement(CS_LexicalBlockStatement &lbs) {
  parser.parse(lbs.lexicalBlockDeclarationStore());
  CS_DepthFirstVisitor::visitLexicalBlockStatement(lbs);
}

