/***********************************************************************/
/*                                                                     */
/* RIGI Utilities --- Copyright (C) 1996-1999                          */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Author: Johannes Martin (jmartin@csr.csc.uvic.ca)                   */
/*                                                                     */
/***********************************************************************/
// make sure error messages stay up before windows disappear...

void PauseBeforeQuitting()
{
  cout << "Type any key and return to quit." << endl;
  char c;
  cin >> c;
}

// Given a return code returned by CS_CodeStore::createTransientDatabase(),
// report the type of error that occurred.

void PrintCodeStoreError(CS_CodeStore::ReturnValue r) {
  cerr << "Error creating codestore:";
  switch (r) {
  case  CS_CodeStore::success:
    cerr << "No error";
    break;
  case CS_CodeStore::readOnly:
    cerr << "readOnly";
    break;
  case CS_CodeStore::transient:
    cerr << "transient";
    break;
  case CS_CodeStore::invalid:
    cerr << "invalid";
    break;
  case CS_CodeStore::incompatible:
    cerr << "incompatible";
    break;
  case CS_CodeStore::accessDenied:
    cerr << "accessDenied";
    break;
  case CS_CodeStore::writeFailure:
    cerr << "writeFailure";
    break;
  case CS_CodeStore::failure:
    cerr << "failure";
    break;
  default:
    cerr << "unknown error";
    break;
  }
}

// PrintMessages handles printing a list of CS_UserMessages.  If
// messages have submessages, then the submessages are printed on
// subsequent lines, indented SUBMESSAGE_INDENTATION spaces.
// (Submessages are extra information that may or may not be useful
// when interpreting errors.)  In the VisualAge C++ GUI, submessages
// in the "Messages" page are displayed in a collapsible list below
// each message.

const SUBMESSAGE_INDENTATION = 4;

void PrintMessages(CS_UserMessage *firstMessage,int indent=0) {
  for (CS_UserMessage *msg = firstMessage ; msg; msg = msg->next()) {
    // put proper indenting on this message.
    for (int i=0;i<indent;i++) cerr << " ";

    // Print file, line, column information
    cerr << msg->messagePrefix();
    cerr << msg->message();
    cerr << endl;

    if (msg->firstSubMessage() != NULL) {
      PrintMessages(msg->firstSubMessage(),indent+ SUBMESSAGE_INDENTATION);
    }
  }
}
