/***********************************************************************/
/*                                                                     */
/* RIGI C Parser --- Copyright (C) 1996-1999                           */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Maintainer: Johannes Martin (jmartin@csr.csc.uvic.ca)               */
/*                                                                     */
/***********************************************************************/
#ifndef __LIST_H__
#define __LIST_H__

template <class T> class List {
  class ListNode {
  public:
    T data;
    ListNode* prev;
    ListNode* next;
    ListNode(const T& d, ListNode *p, ListNode *n) :
      data(d), prev(p), next(n) { };
  };

  ListNode *first;
  ListNode *last;
public:
  List() { first = last = 0; }
  ~List();

  void addFront(const T& element);
  void addTail(const T& element);
  void remFront();
  void remTail();
  const T& getFront() const { return first->data; }
  const T& getTail() const { return last->data; }
  bool empty() { return first == 0; }
};

template <class T> List<T>::~List() {
  while (first != 0) {
    last = first;
    first = first->next;
    delete last;
  }
}

template <class T> void List<T>::addFront(const T& element) {
  ListNode* newNode = new ListNode(element, 0, first);
  if (first != 0)
    first->prev = newNode;
  first = newNode;
  if (last == 0)
    last = newNode;
}

template <class T> void List<T>::addTail(const T& element) {
  ListNode* newNode = new ListNode(element, last, 0);
  if (last != 0)
    last->next = newNode;
  last = newNode;
  if (first == 0)
    first = newNode;
}

template <class T> void List<T>::remFront() {
  if (first == 0)
    return;
  ListNode* temp = first;
  if (first == last)
    first = last = 0;
  else {
    first = first->next;
    first->prev = 0;
  }
  delete(temp);
}

template <class T> void List<T>::remTail() {
  if (last == 0)
    return;
  ListNode* temp = last;
  if (first == last)
    first = last = 0;
  else {
    last = last->prev;
    last->next = 0;
  }
  delete(temp);
}

#endif
