/***********************************************************************/
/*                                                                     */
/* RIGI C Parser --- Copyright (C) 1996-1999                           */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Maintainer: Johannes Martin (jmartin@csr.csc.uvic.ca)               */
/*                                                                     */
/***********************************************************************/
// $Author: jmartin $
// $Date: 2000/08/13 22:29:29 $
// $Source: /home/rigi/proj/cvs/Rigi/rigiparse/cinit.h,v $
// $Locker:  $
// $State: Exp $
//
// ------------------------------------------------------------------------

#ifndef __CINIT_H__
#define __CINIT_H__

#include <stdio.h>
#include <string.h>

// Structure that is passed around on the parse stack.

#define STORAGE_NONE     0
#define STORAGE_AUTO     1
#define STORAGE_REGISTER 2
#define STORAGE_STATIC   3
#define STORAGE_EXTERN   4

class ParseInfo {
  public:
    ParseInfo()
    {
//        fprintf(stderr,"ParseInfo::ParseInfo()\n");
        type = name = qualifiedName = file = NULL;
	line = col = 0;
	storage = STORAGE_NONE;
	pointer = false;
	function = false;
    }
    ParseInfo(const char *n, const char* t, const char* q,
	      const char* f, int l, int c)
    {
//        fprintf(stderr,"ParseInfo::ParseInfo()\n");
        type = name = qualifiedName = file = NULL;
        SetName(n);
        SetType(t);
	SetQualifiedName(q);
	SetFile(f);
	SetLine(l);
	SetCol(c);
	SetPointer(false);
	SetFunction(false);
	storage = STORAGE_NONE;
    }
    ParseInfo(const ParseInfo &info)
    {
//        fprintf(stderr,"ParseInfo::ParseInfo(ParseInfo&)\n");
        type = name = qualifiedName = file = NULL;
        SetType(info.type);
        SetName(info.name);
	SetQualifiedName(info.qualifiedName);
	SetFile(info.file);
	SetLine(info.line);
	SetCol(info.col);
	SetPointer(info.pointer);
	SetFunction(info.function);
	storage = info.storage;
    }
    ~ParseInfo()
    {
//        fprintf(stderr,"ParseInfo::~ParseInfo()\n");
        if (name) delete[] name;
	if (qualifiedName) delete[] qualifiedName;
        if (type) delete[] type;
	if (file) delete[] file;
    }
    ParseInfo& operator=(const ParseInfo &info)
    {
//        fprintf(stderr,"ParseInfo::operator=\n");
        SetType(info.type);
        SetName(info.name);
	SetQualifiedName(info.qualifiedName);
	SetFile(info.file);
	SetLine(info.line);
	SetCol(info.col);
	SetPointer(info.pointer);
	SetFunction(info.function);
	storage = info.storage;
	return *this;
    }
    void SetType(const char* t)
    {
//        fprintf(stderr,"ParseInfo::SetType(%s)\n",t);
        if (type) delete[] type;
        if (t) {
            type = new char[strlen(t) + 1];
            strcpy(type, t);
        } else type = NULL;
    }
    void SetName(const char* n)
    {
//        fprintf(stderr,"ParseInfo::SetName(%s)\n",n);
        if (name) delete[] name;
        if (n) {
            name = new char[strlen(n) + 1];
            strcpy(name, n);
        } else name = NULL;
    }
    void SetQualifiedName(const char* n)
    {
        if (qualifiedName) delete[] qualifiedName;
        if (n) {
            qualifiedName = new char[strlen(n) + 1];
            strcpy(qualifiedName, n);
        } else qualifiedName = NULL;
    }
    void SetFile(const char* f)
    {
//        fprintf(stderr,"ParseInfo::SetFile(%s)\n",f);
        if (file) delete[] file;
        if (f) {
            file = new char[strlen(f) + 1];
            strcpy(file, f);
        } else file = NULL;
    }
    void SetLine(int l)
    {
        line = l;
    }
    void SetCol(int c)
    {
        col = c;
    }
    void SetStorage(int storage)
    {
        this->storage = storage;
    }
    void SetPointer(bool pointer) 
    {
        this->pointer = pointer;
    }
    void SetFunction(bool function) 
    {
        this->function = function;
    }
    void SetPrototype(bool prototype)
    {
        this->pointer = !prototype;
        this->function = prototype;
	if (prototype)
	  storage = STORAGE_EXTERN;
    }
    const char *Name() const
    {
//        fprintf(stderr,"ParseInfo::Name\n");
        return name;
    }
    const char *QualifiedName() const
    {
      return qualifiedName;
    }
    const char *Type() const
    {
//        fprintf(stderr,"ParseInfo::Type\n");
        return type;
    }
    const char *File() const
    {
//        fprintf(stderr,"ParseInfo::File\n");
        return file;
    }
    int Storage() const
    {
        return storage;
    }
    int Line() const { return line; }
    int Col() const { return col; }
    bool Pointer() const { return pointer; }
    bool Function() const { return function; }
    bool Prototype() const { return function && !pointer; }
    void Print(const char *str) const
    {
        fprintf(stderr,"%s: name = %s, type = %s\n",str,qualifiedName,type);
    }
    void MakeName();
  private:
    char* name, *qualifiedName;
    char* type;
    char* file;
    int   line;
    int   col;
    int   storage;
    bool  pointer;
    bool  function;
};

int findtoken( const char* name );

int newtype( const char *name, int value );

#endif
