/* $Author: jmartin $                                                  
 * $Date: 2000/08/16 21:55:57 $
 * $Source: /home/rigi/proj/cvs/Rigi/cobparse/coblex.l,v $
 * $Locker:  $
 * $State: Exp $
 *
 * ------------------------------------------------------------------------
 */

%{
/*									*/
/*		LEX generated COBOL lexical analyser			*/
/*									*/
#include <stream.h>
#include <string.h>
#include "cobparse.h"
#include "cobinit.h"
#include "globals.h"
#include "coblex.h"

int lex_column;
int lex_line;
const char *lex_file;
int garbage_mode = 0;

#define	yywrap() (1)

////////////////////////////////////////////////////////////////////////////

static int ident();
static int checkt(int);
static void count();
static void strings(int);
static void comment();

static const int BUFFSIZE = 64;
static char strbuf[BUFFSIZE];

static void inline load_buf( int c, int i )
	{ if (i<BUFFSIZE-1) strbuf[i]=c; }
static inline char *newstring( char s[] )
	{ return strdup((const char*)s); }

#define COUNTCHECK	{ int i = lex_column; count(); if (ignoreFirst7 && (i < 7)) { comment(); YY_BREAK; } } 
#define RETURNTOKEN(x)	{ COUNTCHECK; return checkt(x); }

%}

D			[0-9]
NUM			[-+]?({D}+|({D}+"."{D}+)|("."{D}+))([eE][-+]?{D}+)?[lL]?
ID			[A-Za-z0-9]([A-Za-z0-9_]|"-")*
SPACE			[ \n\t\v\f\r\x1a]
%%

NOT{SPACE}+AT		{ RETURNTOKEN(TNOTAT); }
NOT{SPACE}+INVALID	{ RETURNTOKEN(TNOTINVALID); }
EJECT			{ COUNTCHECK; }
EJECT\.			{ COUNTCHECK; }
{NUM}			{ COUNTCHECK; yylval.name = newstring(yytext); 
			  return checkt(TNLIT); }
{ID}			{ RETURNTOKEN(ident()); }
")"			{ RETURNTOKEN(TRPARENS) ; }
"("			{ RETURNTOKEN(TLPARENS) ; }
";"			{ RETURNTOKEN(TSEMI) ; }
"."			{ RETURNTOKEN(TPERIOD) ; }
","			{ RETURNTOKEN(TCOMMA) ; }
"+"			{ RETURNTOKEN(TPLUS); }
"-"			{ RETURNTOKEN(TMINUS); }
"**"			{ COUNTCHECK; 
			  if (lex_column == 8) 
			    comment(); 
			  else 
			    return checkt(TEXP); 
			}
"*"			{ COUNTCHECK; 
			  if (lex_column == 7) 
			    comment(); 
			  else 
			    return checkt(TSTAR); 
			}
"/"			{ if (ignoreJCL && (lex_column == 0)) { 
			    comment(); 
			    return(TJCL); 
			  } else 
			    RETURNTOKEN(TSLASH); 
			}
">="			{ RETURNTOKEN(TGE); }
">"			{ RETURNTOKEN(TGT); }
"<="			{ RETURNTOKEN(TLE); }
"<"			{ RETURNTOKEN(TLT); }
"="			{ RETURNTOKEN(TEQ); }

"\""			{ COUNTCHECK; 
			  strings('"'); yylval.name = strbuf;
			  return checkt(TALIT); }
"'"			{ COUNTCHECK; 
			  strings('\''); yylval.name = strbuf;
			  return checkt(TALIT); }

{SPACE}			{ count(); }
.			{ RETURNTOKEN(yytext[0]); }

%%

static int checkt( int token )
{
	if(debug)fprintf(stdout,"Returning %d (%s)\n",token,yytext);
	return token;
}

static int ident()
{
	// an identifier or a reserved word
	int token = findsym( (char*)yytext,extended );
	if (token==TID) yylval.name = newstring( yytext );
	return token;
}

static void count()
{
	int i;

	if (lex_column == 0) fflush(yyout);

	for ( i=0; yytext[i] != '\0'; i++ )
		if (yytext[i] == '\n') {
			lex_column = 0;
			lex_line++ ;
		}
		else if (yytext[i] == '\t') {
			lex_column += 8 - (lex_column % 8);
		}
		else {
			lex_column++;
		}
}

static void comment ()
{
	// swallow the rest of a comment

	register int	c = yyinput();
	register int	startline = lex_line;

	while	(c != '\n')
	{
		c = yyinput() ;
	}
	lex_column = 0 ;
	lex_line++ ;
}

static void strings( int term )
{
	// A string terminating with 'term'
	// term is allowed to escape itself
	// newline => look for next occurrence of term

	register int c;
	register int i=0;

	load_buf( term, i ); i++;

	while (c = yyinput())
	{
		load_buf( c, i ); i++;
		lex_column++;

		if (c == term) {
			c = yyinput();
			lex_column++;
			if (c!=term) { unput(c); lex_column--; break; }
		}
		else if	(c == '\\') { c = yyinput(); lex_column++; }
		else if (c == '\n') {
			lex_line++; lex_column = 0;
			do {
				c = yyinput();
				load_buf( c, i ); i++;
				lex_column++;
			} while (c!=EOF && c!=term);

			if (c == EOF) break;
			continue;
		}
	}
	load_buf( '\0', i );
}

void lex_init (const char *filename)
{
	//  Initialise the Lexical Analyser for a new file
	//  lex itself should provide this!

	lex_column	= 0 ;
	lex_line	= 1 ;
        lex_file        = filename ;
}

// ------------------------------------------------------------------------
//
// $Log: coblex.l,v $
// Revision 1.5  2000/08/16 21:55:57  jmartin
// lots of more improvements (more language features supported)
//
// Revision 1.4  2000/08/15 06:41:27  jmartin
// major overhaul & extensions
//
// Revision 1.3  2000/08/13 22:34:15  jmartin
// configure'able version of cobparse
//
// Revision 1.2  2000/08/13 02:27:26  jmartin
// check-in of mwhitney's changes of September 1995
//
//   Revision 1.1  1994/04/27  22:18:45  mwhitney
//   Initial revision
//
//
