/***********************************************************************/
/*                                                                     */
/* RIGI Utilities --- Copyright (C) 1996-1999                          */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Author: Johannes Martin (jmartin@csr.csc.uvic.ca)                   */
/*                                                                     */
/***********************************************************************/
#ifndef __HTMLTUPL_H__
#define __HTMLTUPL_H__

#include "rsftuple.h"

class HtmlRsfTuple : public RsfTuple {
  typedef set<UniqueString, less<UniqueString> > StringSet;
  static StringSet anchors;
  static StringSet bookmarks;

  inline bool isKeyword(const StringSet& keywords) {
    return keywords.find(rsf[0]) != keywords.end();
  }

public:
  enum HtmlType { NoType, Anchor, Bookmark };

  /*
   * Predefined UniqueStrings for some standard RSF key words.
   */
  static const UniqueString nodeurlString;

  /*
   * html Information for RSF tuple
   */
  UniqueString oFile;
  int	       oLine;
  HtmlType     htmlType;

  /*
   * Create a new HtmlRsfTuple.
   */
  HtmlRsfTuple(const UniqueString& elem0, const UniqueString& elem1,
	       const UniqueString& elem2,
	       const UniqueString& elem3 = UniqueString::emptyString,
	       int line = 0, int column = 0,
	       const UniqueString& oFile = UniqueString::emptyString,
	       int oline = 0);

  /*
   * Create a new HtmlRsfTuple from an RsfTuple.
   */
  HtmlRsfTuple(const RsfTuple& rsfTuple,
	       const UniqueString& oFile = UniqueString::emptyString,
	       int oline = 0);

  /*
   * Parse a line of an RSF file and put its elements into an RsfTuple.
   * Remove the 4th element in the line, if preserve4Tuple is not set.
   * Errors will be printed to stderr. Check isValid() to see whether
   * the tuple was actually created.
   */
  HtmlRsfTuple(char *inputLine, unsigned long lineNo, bool preserve4Tuple);

  /*
   * Compare two tuples by comparing type and name. 'type' tuples come
   * first, 'loc' tuples last, in between, tuples are sorted by their
   * second element (third, first, fourth element as secondary sort keys).
   */
  bool operator< (const HtmlRsfTuple& other) const;

  /*
   * Print out an RsfTuple to stdout. If the tuple is a bookmark,
   * also print a "nodeurl" RsfTuple.
   * Remove the 4th element in the line, if preserve4Tuple is not set.
   * Return false if printf failes, otherwise true.
   */
  bool print(FILE* output, bool preserve4Tuple = false) const;

  /*
   * Add a string that marks an anchor/bookmark.
   */
  static void addAnchorString(const UniqueString& s);
  static void addBookmarkString(const UniqueString& s);
};

#endif
