// $Author: jmartin $
// $Date: 2000/08/17 19:08:36 $
// $Source: /home/rigi/proj/cvs/Rigi/cobparse/main.C,v $
// $Locker:  $
// $State: Exp $
//
// ------------------------------------------------------------------------
// MJW: COBOL parse - converts to RSF triples

#include <stdio.h>
#include <iostream.h>
#include <stdlib.h>		// atoi() and exit()
#include <string.h>		// rindex()

#include "globals.h"
#include "coblex.h"

// interface to parsing stuff
extern FILE *yyin;
int yyparse();
#if YYDEBUG != 0
extern "C" int yydebug;
#endif

void Help(const char* execName)
{
  cerr << PRODUCT " - " VERSION "\n";
  cerr << "usage: " << execName << "[options] [<inputFile>]\n";
  cerr << "\t-d\tdebug\n";
  cerr << "\t-k\tuse only the original reserved keywords\n";
  cerr << "\t-q\tquiet: no relations output\n";
  cerr << "\t-s(n)\tstop at first (or nth) syntax error\n";
  cerr << "\t-4\tcreate 4-element RSF file\n";
  cerr << "\t-7\tignore all lines starting in columns 1-7\n";
  cerr << "\t-j\tignore all lines with '/' in column 1 (JCL)\n";
#if YYDEBUG != 0
  cerr << "\t-y\tset yydebug (for yacc debugging)\n";
#endif
  cerr << "\t-h\thelp: print this\n";
}

static int Parse(const char* filename)
{
  int r;

  if (!filename) {
    yyin = stdin;
    filename = "<stdin>";
  } else {
    if ((yyin = fopen( filename,"r" )) == NULL) {
      cerr << PRODUCT ": error - cannot open input file ";
      cerr << filename << "\n";
      exit(1);
    }
    cerr << PRODUCT ": starting to parse file \"" << filename << "\"\n";
  }
  
  lex_init( filename );
  compiler_init();
  
  r = yyparse();
  
  if (r) 
    cerr << PRODUCT ": parsing failed\n";
  return r;
}

void main( int argc, char **argv )
{
  char *filename = NULL;
  char *execName = argv[0];
 
  // process arguments
  while (--argc)
    if (**++argv=='-') 
      switch (*(*argv+1)) {
      case 'd':	
	debug = 1;	
	break;
      case 'h':	
	Help(execName);	  
	exit(0);
      case 'k':	
	extended = 0;	
	break;
      case 'q':	
	quiet = 1;	
	break;
      case 's':	
	syntax_stop = atoi( *argv+2 );
	if (!syntax_stop) 
	  syntax_stop = 1;
	break;
#if YYDEBUG != 0
      case 'y':	
	yydebug = 1;	
	break;
#endif
      case '4':
	write4 = 1;
	break;
      case '7':
	ignoreFirst7 = 1;
	break;
      case 'j':
	ignoreJCL = 1;
	break;
      } 
    else 
      filename = *argv;

  return(Parse(filename));
}

// ------------------------------------------------------------------------
//
// $Log: main.C,v $
// Revision 1.6  2000/08/17 19:08:36  jmartin
// small changes and documentation
//
// Revision 1.5  2000/08/16 21:55:57  jmartin
// lots of more improvements (more language features supported)
//
// Revision 1.4  2000/08/15 06:41:28  jmartin
// major overhaul & extensions
//
// Revision 1.3  2000/08/13 22:34:16  jmartin
// configure'able version of cobparse
//
// Revision 1.2  2000/08/13 02:27:26  jmartin
// check-in of mwhitney's changes of September 1995
//
//   Revision 1.1  1994/04/27  22:18:45  mwhitney
//   Initial revision
//
//
