/***********************************************************************/
/*                                                                     */
/* RIGI C Parser --- Copyright (C) 1996-1999                           */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Maintainer: Johannes Martin (jmartin@csr.csc.uvic.ca)               */
/*                                                                     */
/***********************************************************************/

// MJW: Rigi parse driver - like a new version of rigireverse.
// Quite rudimentary right now. Only for C parsing. It grabs
// the appropriate preprocessor out of RIGICPP

#include <stdio.h>
#include <string.h>		// for strrchr
#include <stdlib.h>		// for getenv
#include <unistd.h>             // for unlink

#include <string>

static void Help(char *execName, char* parsername) {
  fprintf( stderr, "rigiparse - " VERSION "\n");
  fprintf( stderr, "\nusage: %s [rigiparse options] [child parser options]\n",
	   execName);
  fprintf( stderr,"rigiparse options are:\n");
  fprintf( stderr,"\t-h\t\tprint this help\n");
  fprintf( stderr,"\t-o file\t\tuse file for RSF output\n");
  fprintf( stderr,"\t-p<parser>\tparser to use\n");
  fprintf( stderr,"\t-w\t\twait for user input before parsing\n");
  fprintf( stderr,"\t-I<path>\tpath for include files (passed to RIGICPP)\n");
  fprintf( stderr,"\t-D<define>\tC macro definition (passed to RIGICPP)\n");
  fprintf( stderr,"\nchild parser options for %s are:\n", parsername);

  char *parsecommand = (char*) malloc(strlen(parsername) + 4);
  strcpy(parsecommand, parsername);
  strcat(parsecommand, " -h");
  system(parsecommand);
}

int main( int argc, char **argv ) {
  int waituser     = 0;
  char *parsername = "cparse";
  char *execName   = *argv++;
  char *ofile      = NULL;
  string parseropt;
  string preprocessor;

  if (!getenv( "RIGICPP" )) {
#ifdef _MSC_VER
    preprocessor = "cl -E";
#else
    preprocessor = "cc -E";
#endif
    fprintf(stderr, "%s: RIGICPP is undefined; preprocessor is %s\n",
	    execName, preprocessor.c_str() );
  } else 
    preprocessor = getenv("RIGICPP");

  // Process arguments. All args that are not recognized as
  // rigiparse options are passed on to the child parser,
  // EXCEPT -I and -D options, which go to the preprocessor.

  while ((--argc > 0) && (**argv == '-')) {
    switch ((*argv)[1]) {
    case 'h': 
      Help(execName, parsername); 
      exit(0); 
    case 'p': 
      parsername = strdup(*argv + 2); 
      break;
    case 'w': 
      waituser = 1; 
      break;
    case 'o':
      ofile = *++argv; 
      argc--;
      break;
    case 'D':
    case 'I':
      // preprocessor options
      preprocessor += ' ';
      preprocessor += *argv;
      break;
    default:
      // Child parser options ...
      parseropt += ' ';
      parseropt += *argv;
      break;
    }
    argv++;
  }

  if (ofile)
    unlink(ofile);

  // interpret remaining arguments as files

  int i   = 0;
  int rc  = 0;

  while (argc-->0) {
    if ((*argv)[0] == '-') {
      fprintf( stderr,"%s: error, bad syntax, option %s\n", execName, *argv );
      exit(1);
    }

    string command = preprocessor + " " + *argv + " | " + parsername + parseropt;
    if (ofile) {
      command += " >> ";
      command += ofile;
    }
    fprintf(stderr,"%s: %s\n", execName, command.c_str());
    i++;

    if (waituser) {
      printf("press enter to continue:");
      fflush(stdout);
      getchar();
    }
    rc = system( command.c_str());
    if (rc) {
      fprintf(stderr,"%s: parse of file %s failed\n", execName, argv);
      break;
    }
    argv++;
  }

  fprintf( stderr,"%s: %d file(s) parsed\n", execName, i );
  exit(rc);
}
