/***********************************************************************/
/*                                                                     */
/* RIGI Utilities --- Copyright (C) 1996-1999                          */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Author: Jim McDaniel                                                */
/*                                                                     */
/***********************************************************************/
/* sortrsf.cpp (sort/filter RSF)

   Sorts RSF file, placing "type"'s at the top of the file.
   Removes duplicate tuples and, if -m is specified, coalesces
   tuples with same second and third components into a
   single "multiarc" tuple.  If -l is specified, the program
   combines loc attributes referencing the same object by
   concatenating the references.  If -4 is specified, the
   program preserves 4-tuples (otherwise, 4th element will be
   discarded on output).
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "uniqstr.h"
#include "rsffile.h"

enum ArgType {
  ArgNone, ArgError, ArgHelp, ArgMultiarc = 4, ArgLoc = 8, Arg4 = 16
};

static int ParseCmdLine(int argc, char **argv) {
  int i, j, k, args = 0;
  char c;

  for (i = 1; i < argc; i++) {
    if (*argv[i] != '-') {
      fprintf(stderr, "Invalid argument \"%s\"\n", argv[i]);
      return (ArgError | ArgHelp);
    }
    k = strlen(argv[i]);
    for (j = 1; j < k; j++) {
      c = *(argv[i] + j);
      switch (c) {
      case 'h':
	args |= (int)ArgHelp;
	break;
      case 'm':
	args |= (int)ArgMultiarc;
	break;
      case 'l':
	args |= (int)ArgLoc;
	break;
      case '4':
	args |= (int)Arg4;
	break;
      default:
	fprintf(stderr, "Invalid argument \"%s\"\n", argv[i]);
	return (ArgError | ArgHelp);
      }
    }
  }
  return args;
}

static void PrintHelp(char *progName) {
  fprintf(stderr,
	  "Rigi Standard Format Sort and Filter (sortrsf) - " VERSION "\n");
  fprintf(stderr,
	  "Usage: %s [-hlm4]\n", progName);
  fprintf(stderr,
	  "\t-h help\n");
  fprintf(stderr,
	  "\t-l combine multiple \"loc\" attributes that reference the same object\n");
  fprintf(stderr,
	  "\t-m combine duplicate arcs into single \"multiarc\" type\n");
  fprintf(stderr,
	  "\t-4 preserve 4-tuples (overrides -l and -m)\n");
}

int main(int argc, char *argv[]) {
  int args = 0;

  if (argc > 1) {
    args = ParseCmdLine(argc, argv);
    if (args & ArgHelp) {
      PrintHelp(argv[0]);
      return (args & ArgError);
    }

    /* If sorting 4-tuples, override other switches */
    if (args & Arg4)
      args = Arg4;
  }

  set<RsfTuple, less<RsfTuple> > tuples;
  if (!readInput(tuples, args & Arg4))
    return 1;
  return !writeOutput(tuples, args & Arg4, args & ArgLoc, args & ArgMultiarc);
}
