/***********************************************************************/
/*                                                                     */
/* RIGI Utilities --- Copyright (C) 1996-1999                          */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Author: Johannes Martin (jmartin@csr.csc.uvic.ca)                   */
/*                                                                     */
/***********************************************************************/
#ifndef __UNIQSTR_H__
#define __UNIQSTR_H__

#include <string>
#include <set>

#ifndef __GNUG__
using std::string;
using std::less;
using std::set;
#endif

typedef string String;

class UniqueString {
  static set<String, less<String> > strings;

  const String* theString;

  UniqueString(int) {
    theString = &(*strings.insert("").first);
  }
public:
  const static UniqueString emptyString;

  UniqueString() {
    theString = emptyString.theString;
  }

  UniqueString(const char* s) {
    *this = s;
  }

  UniqueString(const String& s) {
    *this = s;
  }

  UniqueString(const UniqueString& s) {
    theString = s.theString;
  }

  UniqueString& operator=(const char* s) {
    return *this = String(s);
  }

  UniqueString& operator=(const String& s) {
    if (s.empty())
      theString = emptyString.theString;
    else
      theString = &(*strings.insert(s).first);
    return *this;
  }

  UniqueString& operator=(const UniqueString& s) {
    theString = s.theString;
    return *this;
  }

  bool operator==(const UniqueString& s) const {
    return theString == s.theString;
  }

  bool operator!=(const UniqueString& s) const {
    return !operator==(s);
  }

  bool operator<(const UniqueString& s) const {
    return string() < s.string();
  }

  int empty() const {
    return theString == emptyString.theString;
  }

  operator const String&() const {
    return string();
  }

  const String& string() const {
    return *theString;
  }

  operator const char*() const {
    return chars();
  }

  const char* chars() const {
    return theString->c_str();
  }

  int hash() const {
    return (int) theString;
  }
};

#endif
